#!/usr/bin/env python
# -*- coding: utf_8 -*-
#
# @author  killdream <killdreamxd@gmail.com>
# @version 0.2.1 alpha
#
# Copyright (C) 2008 Quildreen <killdreamxd@gmail.com>
# -----                                                   
# This program is free software; you can redistribute  
# it and/or modify it under the terms of the GNU       
# General Public License as published by the Free      
# Software Foundation; either version 2 of the License,
# or (at your option) any later version.               
#                                                         
# This program is distributed in the hope that it will 
# be useful, but WITHOUT ANY WARRANTY; without even    
# the implied warranty of MERCHANTABILITY or FITNESS   
# FOR A PARTICULAR PURPOSE. See the GNU General Public 
# License for more details.                            
#                                                         
# You should have received a copy of the GNU General   
# Public License along with this program; if not,      
# write to the Free Software Foundation, Inc., 675     
# Mass Ave, Cambridge, MA 02139, USA.

# Import pygame modules
import pygame
from pygame.locals import *

import sprite
import core
import math

class oBasicAttack:
    def __init__(self, image, **kwargs):
        self.image = image
        self.power = kwargs.pop("power", 1)
        self.freeze = 0
        self.feathers = kwargs.pop("feathers", -1)
        self.used = 0
        self.kills = 0
        self.waste_penalty = 10
        self.last_rects = []
        
    def attack(self, char):
        if self.freeze > 0 or self.feathers == 0:
            return
        
        self.used += 1
        
        core.play_sound("feather_throw")
        feather = sprite.oSprite()
        feather.attackMgr = self
        tw, th = core.level.tileSize
        feather.speed = .25
        feather.direction = char.direction
        if char.direction == "N":
            feather.y -= th/2
        if char.direction == "S":
            feather.y += th/2
        if char.direction == "E":
            feather.x += tw/2
        if char.direction == "W":
            feather.x -= tw/2
            
        feather.life = 700
        feather.AI = featherAI
        feather_char = core.load_image("feather.png", directory="gfx")
        feather.animFromStrip( feather_char, [
                ["stopped","E",0,1,0,True, (3, 15)],
                ["stopped","N",1,1,0,True, (5,13)],
                ["stopped","W",2,1,0,True, (12,15)],
                ["stopped","S",3,1,0,True, (5,11)]
            ], 24, 24, feather_char)
        feather.changeState("stopped")
        feather.x = char.rx() + char.action_point()[0] - feather.hotspot()[0]
        feather.y = char.ry() + char.action_point()[1] - feather.hotspot()[1]
        core.attack_group.add(feather)
        core.render_group.add(feather)
        self.freeze = 250
        if self.feathers != -1:
            self.feathers -= 1
            if self.feathers < 0:
                self.feathers = 0
        
    def update(self, lag):
        self.freeze -= lag
        
    def draw(self, surface, active):
        rects = [pygame.Rect(720, 520, 70, 70)]
        feathers = ""
        fw = 0
        fh = core.numbers.height
        if self.feathers != -1:
            feathers = str(min(self.feathers, 99))[:2].zfill(2)
        else:
            feathers = "!"
        fw = core.numbers.getWidth( feathers )
        rects.append( pygame.Rect(720-fw/2, 590-fh, fw, fh) )
        
        if active:
            surface.blit( self.image, (720,520) )
            core.numbers.draw( surface, (720-fw/2, 590-fh), feathers )
            
        last_rect = self.last_rects[:]
        self.last_rects = rects[:]
        rects.extend(last_rect)
        return rects
        
def featherAI(feather, lag):
    if feather.direction == "N":
        feather.y -= feather.speed*feather.speedMul * lag
    if feather.direction == "S":
        feather.y += feather.speed*feather.speedMul * lag
    if feather.direction == "E":
        feather.x += feather.speed*feather.speedMul * lag
    if feather.direction == "W":
        feather.x -= feather.speed*feather.speedMul * lag
        
    # verify if we hit a nest :D
    if not feather.canWalk():
        w, h = core.level.tileSize
        x = int( math.floor( feather.rx() / w ) )
        y = int( math.floor( (feather.ry()-33) / h ) )# fix tiles have a blank area on top o.o
        tile = core.level.getTile(x, y)
        if tile is not None and tile.nest is not None:
            tile.nest.doDamage(feather.attackMgr.power)
            feather.attackMgr.kills += 1
            feather.dead = True
        
    feather.life -= lag
    if feather.life < 0:
        feather.dead = True
        
        
        