#!/usr/bin/env python
# -*- coding: utf_8 -*-
#
# @author  killdream <killdreamxd@gmail.com>
# @version 0.2.1 alpha
#
# Copyright (C) 2008 Quildreen <killdreamxd@gmail.com>
# -----                                                   
# This program is free software; you can redistribute  
# it and/or modify it under the terms of the GNU       
# General Public License as published by the Free      
# Software Foundation; either version 2 of the License,
# or (at your option) any later version.               
#                                                         
# This program is distributed in the hope that it will 
# be useful, but WITHOUT ANY WARRANTY; without even    
# the implied warranty of MERCHANTABILITY or FITNESS   
# FOR A PARTICULAR PURPOSE. See the GNU General Public 
# License for more details.                            
#                                                         
# You should have received a copy of the GNU General   
# Public License along with this program; if not,      
# write to the Free Software Foundation, Inc., 675     
# Mass Ave, Cambridge, MA 02139, USA.

# import system modules
import os, sys

# Import pygame modules
import pygame
from pygame.locals import *

import map
import core
import sprite
import fontmgr
from evinput import Input
import featheratk

mc = None

class oStatsDisp:
    def __init__(self, iValue, position, **kwargs):
        self.value = iValue
        self.x, self.y = position
        self.xalign= kwargs.pop("xalign", 0)
        self.width = kwargs.pop("width", 800)
        self.image = kwargs.pop("image", None)
        self.font  = kwargs.pop("font" , None)
        self.size  = kwargs.pop("size" , None)
        self.aalias= kwargs.pop("antialias", True)
        self.color = kwargs.pop("color", (255,255,255,255))
        self.useNumbers = kwargs.pop("useNumbers", False)
        self.changed = True
        self.formatValue = kwargs.pop("formatValue", str)
        self.last_rects = [ ]
        
    def update(self, iValue, forceUpdate=False):
        if iValue != self.value or forceUpdate:
            self.value = iValue
            self.changed = True
            
    def draw(self, surface):
        rects = [ ]
        # assume we have a image to draw for each value =D
        if self.image is not None:
            w = self.image.get_width()
            for i in range(self.value):
                rects.append( surface.blit(self.image, (self.x + w*i, self.y)) )
        
        # otherwise we just renderize a font for it =D
        else:
            if self.useNumbers:
                x = self.x + (self.xalign * (self.width-core.numbers.getWidth(self.formatValue(self.value))))
                    
                rect = core.numbers.draw(surface, (x, self.y), self.formatValue(self.value))
            else:
                rect = core.fonts.Draw(surface, self.font, self.size, self.formatValue(self.value),
                (self.x, self.y), self.color, antialias=self.aalias) 
                
            rects.append(rect)
             
        lr = self.last_rects[:]
        self.last_rects = rects[:]
        rects.extend(lr)
        return rects
    
class oPortalEquation():
    def __init__(self, mapa, pos):
        self.equation = []
        self.paths    = []
        self.portalN  = mapa.portalN
        self.terms    = 0
        self.state    = 0
        self.changed  = False
        self.mapa     = mapa
        self.x, self.y= pos
        self.last_rect= None
        self.loadData()
        
    def loadData(self):
        for index, tile in enumerate(self.mapa.mapData):
            if tile is not None:
                # path
                if tile.op == -1 and tile.number > -1 and not tile.number in self.paths:
                    self.paths.append(tile.number)
                # term
                elif tile.op > -1 and tile.number > -1:
                    self.terms += 1
                    
    def activateTerm(self, x, y):
        index = y * self.mapa.width + x
        tile  = self.mapa.getTile(x, y)
        # check if it's already activated (ugly, I know)
        for term in self.equation:
            if term[0] == index:
                self.equation.remove(term)
                break
            
        if tile.active:
            self.equation.append( [index, tile.number, tile.op] )
            
        ee = self.getResult()
        if ee == self.portalN:
            self.state = 2
        elif ee in self.paths:
            self.state = 1
        else:
            self.state = 0
        self.changed = True

    def evalEquation(self):
        result  = ""
        last_op = ""
        for term in self.equation:
            index, number, op = term
            result += last_op + str(number)
            last_op = core.OP_TABLE[op]
        return result
    
    def getResult(self):
        r = self.evalEquation()
        if r == "":
            return -1
        else:
            return eval(r)
    
    def draw(self, surface):
        color = core.EQ_COLOR_TABLE[self.state]
        eq = str(self.portalN) + " = " + self.evalEquation()
        if len(self.equation) == 0:
            eq += "?"
        
        rect = core.numbers.draw( surface, (self.x, self.y), eq )
        lr = self.last_rect
        self.last_rect = rect
        return [lr, rect]

def load_level(levelN, mc):
    core.baddys_group.clear()
    core.attack_group.clear()
    if os.path.exists("map/level_" + str(levelN) + ".rmap"):
        core.play_level_music(levelN)
        core.level.load("map/level_" + str(levelN) + ".rmap")
        core.mathExp = oPortalEquation(core.level, (20, 545))
        px, py = core.level.findPortal()
        tw, th = core.level.tileSize
        mc.x = px*tw + mc.hotspot()[0]
        mc.y = py*th + mc.hotspot()[1]
        
        if levelN == 0:
            core.level.nests = [
                map.oNest(3, 11, "S", core.level, core.baddys_group),
                map.oNest(5, 116, "E", core.level, core.baddys_group)
            ]
        if levelN == 1:
            core.level.nests = [
                map.oNest(2, 0, "S", core.level, core.baddys_group),
                map.oNest(2, 2, "S", core.level, core.baddys_group),
                map.oNest(4, 113, "N", core.level, core.baddys_group, maxState=2, state=1)
            ]
            
        return True
    else:
        return False

def restart_level(mc):
    px, py = core.level.findPortal()
    tw, th = core.level.tileSize
    mc.x = px*tw + mc.hotspot()[0]
    mc.y = py*th + mc.hotspot()[1]

def format_level_time(value):
    return str( int(value) )

def format_score(value):
    return str(int(value))[:9].zfill(9)
        
def hiscores_screen(score=None, rank=0):
    last_screen = core.screen.copy().convert()
    scores = []
    
    import xdrlib
    if not os.path.exists("high_scores"):
        # fill with default scores
        default_scores = [(100000,2),(50000,2),(30000,2),(20000,2),(10000,2),(8000,1),
                          (4000,1),(500,0)]
        for i in range(8):
            s, r = default_scores[i]
            scores.append( ("Nozomi", s, r) )
    
    else:
        # load from file
        data = open("high_scores", "rb").read()
        loader = xdrlib.Unpacker(data)
        for i in range(8):
            scores.append( (loader.unpack_string(), loader.unpack_int(), loader.unpack_int()) )
        
        
    # verify if we have a high score =D
    if score is not None:
        for index, _score in enumerate(scores):
            name, _score, _rank = _score
            if (score > _score) or (score == _score and rank > _rank):
                player_name = core.InputPrompt("Nice timing! You've scored high, Mr/Mrs...", "Player")
                scores.insert(index, (player_name, score, rank))
                break
    
        # save scores
        p = xdrlib.Packer()
        for name, score, rank in scores:
            p.pack_string(name)
            p.pack_int(score)
            p.pack_int(rank)
            
        f = open("high_scores", "wb")
        f.write(p.get_buffer())
        f.close()
    
    # load graphics =D
    bg = core.load_image("hiscores.png", directory="gfx")
    stars = pygame.transform.smoothscale(core.load_image("rank_star.png", directory="gfx"), (86,40))
    stars = [ core.im_crop(stars, pygame.Rect(0,0,43,40)),
              core.im_crop(stars, pygame.Rect(43,0,43,40)) ]
    
    
    core.screen.blit(bg, (0,0))
    
    
    # render high scores!
    for index, score in enumerate(scores):
        name, score, rank = score
        # 260, 100
        y = 100 + 40*index
        # name
        core.fonts.Draw(core.screen, None, 36, name, pygame.Rect(260,y,200,40),
         (255,255,255,255),alignVert='center',antialias=True)
        
        # points
        score = str(score)
        w = 160-core.numbers.getWidth(score)
        core.numbers.draw(core.screen, (470+w,y+8), score)
        
        # rank
        if rank > 2:
            core.screen.blit(stars[1], (660,y))
        else:
            core.screen.blit(stars[0], (660,y))
        if rank > 1:
            core.screen.blit(stars[1], (703,y))
        else:
            core.screen.blit(stars[0], (703,y))
        if rank > 0:
            core.screen.blit(stars[1], (746,y))
        else:
            core.screen.blit(stars[0], (746,y))
            
    core.dissolve_transition(last_screen, core.screen.copy().convert(), 1000)
    
    while True:
        events = pygame.event.get()
        if core.handleEvents(events) == "quit":
            exit()
            
        for event in events:
            if event.type == KEYUP:
                core.next_screen = main_menu
                core.play_sound("button_click")
                Input.update()
                return
    
    
def getRank(mc, lost_lives, lives, level_time, killed_bugs):
    
    used_feathers, wasted_feathers, waste_penalty = getFeatherStats(mc)
    
    level_time = int(level_time)
    if used_feathers == 0:
        feather_rate = 1
    else:
        feather_rate = 1 - (wasted_feathers / float(used_feathers))
    
    if lives == 0:
        lives_rate = 0
    else:
        lives_rate = lost_lives / float(lives)
        
    if level_time == 0:
        level_rate = 0
    else:
        level_rate = level_time / float(core.level.time-100)
        
    if core.level.created_bugs == 0:
        bugkill_rate = 1
    else:
        bugkill_rate = killed_bugs / float(core.level.created_bugs)
        
    rank = ( feather_rate + lives_rate + level_rate + bugkill_rate) / 4.0
    
    print rank
    
    if rank >= .8:
        return 3
    if rank >= .5:
        return 2
    if rank >= .2:
        return 1
    
    return 0
    
def getFeatherStats(mc):
    used_feathers = 0
    wasted_feathers = 0
    waste_penalty = 0
    for attack in mc.attacks:
        used_feathers   += attack.used
        wasted_feathers += used_feathers - attack.kills
        waste_penalty   += wasted_feathers * attack.waste_penalty
        
    return used_feathers, wasted_feathers, waste_penalty
    
def level_clear_screen(mc, killed_bugs, ltime, lives, lost_lives, score):
    core.play_music("stage clear", queue=True)
    last_screen = core.screen.copy().convert()
    ls = last_screen.copy().convert()
    bg = core.load_image("stage_clear.png", directory="gfx")
        
    stars = core.load_image("rank_star.png", directory="gfx")
    stars = [ core.im_crop(stars, pygame.Rect(0,0,55,51)),
              core.im_crop(stars, pygame.Rect(55,0,55,51)) ]
    
    ltime = int(ltime)
    used_feathers, wasted_feathers, waste_penalty = getFeatherStats(mc)
    rank = getRank(mc, lost_lives, lives, ltime, killed_bugs)
     
    state = 0
    
    print ltime
    skilled_bugs = oStatsDisp(0, (465,155), useNumbers=True, xalign=1.0, width=100, formatValue=format_level_time)
    stime = oStatsDisp(0, (465,190), useNumbers=True, xalign=1.0, width=100, formatValue=format_level_time)
    slives = oStatsDisp(0, (465,225), useNumbers=True, xalign=1.0, width=100, formatValue=format_level_time)
    sfeathers = oStatsDisp(0, (465,260), useNumbers=True, xalign=1.0, width=100, formatValue=format_level_time)
    stotal = oStatsDisp(score, (235,370), useNumbers=True, xalign=1.0, width=330, formatValue=format_score)
    last_total = score
    
    forceUpdate = True
    vel = .3
    vel2 = 100
    
    while True:
        lag = core.clock.tick(core.MAX_FPS)
        if lag <= 0:
            lag = 1
            
        dirty_rects = []
        
        if core.handleEvents() == "quit":
            exit()
            
        vel = 0.2
        if Input.up(K_RETURN):
            if state < 8:
                vel = sys.maxint
            else:
                core.dissolve_transition(core.screen.copy().convert(), ls, 1000)
                return stotal.value, rank
        
        core.screen.blit(ls, (0,0))
        core.screen.blit(bg, (0,0))
        # points
        if state == 0:
            skilled_bugs.update( skilled_bugs.value + vel*lag )
            if skilled_bugs.value >= killed_bugs:
                skilled_bugs.value = killed_bugs
                state += 1
        elif state == 1:
            stime.update( stime.value + vel*lag )
            if stime.value >= ltime:
                stime.value = ltime
                state += 1
        elif state == 2:
            slives.update( slives.value + vel*lag )
            if slives.value >= lives:
                slives.value = lives
                state += 1
        elif state == 3:
            sfeathers.update( sfeathers.value + vel*lag )
            if sfeathers.value >= wasted_feathers:
                sfeathers.value = wasted_feathers
                state += 1
        elif state == 4:
            stotal.update( stotal.value + vel2*lag )
            if stotal.value >= last_total + killed_bugs*10:
                stotal.value = last_total + killed_bugs*10
                last_total = stotal.value
                state += 1
        elif state == 5:
            stotal.update( stotal.value + vel2*lag )
            if stotal.value >= last_total + ltime*100:
                stotal.value = last_total + ltime*100
                last_total = stotal.value
                state += 1
        elif state == 6:
            stotal.update( stotal.value + vel2*lag )
            if stotal.value >= last_total + lives*1000:
                stotal.value = last_total + lives*1000
                last_total = stotal.value
                state += 1
        elif state == 7:
            stotal.update( stotal.value - vel2*lag )
            if stotal.value <= last_total - waste_penalty:
                stotal.value = last_total - waste_penalty
                last_total = stotal.value
                state += 1
        
        # draw
        dirty_rects.extend( skilled_bugs.draw(core.screen) )
        if state > 0:
            dirty_rects.extend( stime.draw(core.screen) )
        if state > 1:
            dirty_rects.extend( slives.draw(core.screen) )
        if state > 2:
            dirty_rects.extend( sfeathers.draw(core.screen) )
        if state > 3:
            dirty_rects.extend( stotal.draw(core.screen) )
        if state > 7:
            def draw_rank_star(pos, rank_need):
                if rank >= rank_need:
                    return core.screen.blit(stars[1], pos)
                else:
                    return core.screen.blit(stars[0], pos)
            
            dirty_rects.append( draw_rank_star((405,430), 3) )
            dirty_rects.append( draw_rank_star((460,430), 2) )
            dirty_rects.append( draw_rank_star((515,430), 1) )
            
        if last_screen is not None:
            core.dissolve_transition(last_screen, core.screen.copy().convert(), 1000)
            last_screen = None
            
        if forceUpdate:
            dirty_rects = None
        forceUpdate = False
        core.updateScreen(dirty_rects)

def main_char_death(mc):    
    def RestartLevel():
        mc.OnAnimationEnd = None
        mc.changeState("stopped")
        restart_level(mc)
    
    mc.godmode = 500
    mc.OnAnimationEnd = RestartLevel
    mc.direction = "N"
    mc.changeState("dying")
    
    
def main_game():    
    global mc
    
    core.baddys_group.clear()
    core.render_group.clear()
    core.attack_group.clear()
    
    # load bg
    sky_bg = core.load_image("sky_bg.png", directory="gfx", auto_alpha=False)
    exp_bgs = core.load_image("exp_bg.png", directory="gfx")
    exp_bg = sprite.oSprite()
    exp_bg.x = 10
    exp_bg.y = 545
    exp_bg.animations = {
        "stopped": {
            "N": [0, True, [
                sprite.oFrame(core.im_crop(exp_bgs, pygame.Rect(0,0,278,33))),
                sprite.oFrame(core.im_crop(exp_bgs, pygame.Rect(0,33,278,33))),
                sprite.oFrame(core.im_crop(exp_bgs, pygame.Rect(0,66,278,33)))
            ]]
        }
    }
    exp_bg.changeState("stopped")
    core.render_group.add(exp_bg)
       
    # initialize map
    core.level = map.oMap()

    # current stats
    level = 0
    level_time = oStatsDisp(core.level.time, (700,20), useNumbers=True, formatValue=format_level_time)
    lives = oStatsDisp(3, (10, 20), image=core.load_image("haato.png", directory="gfx"))
    score = oStatsDisp(0, (0,20), xalign=0.5, useNumbers=True, formatValue=format_score)

    # main character
    mc = sprite.oSprite()
    
    mc_sprite = core.load_image("nozomi.png", directory="gfx")
    mc_mask   = core.load_image("nozomi_mask.png", directory="gfx")
    
    mc.animFromStrip( mc_sprite, [
        ["stopped","S",1,1,0,True, (23,45), (23,33)],
        ["stopped","W",4,1,0,True, (23,45), (17,33)],
        ["stopped","E",7,1,0,True, (23,45), (30,33)],
        ["stopped","N",10,1,0,True,(23,45), (23,33)],
        ["walking","S",0,3,.02,True, (23,45), (23,33)],
        ["walking","W",3,3,.02,True, (23,45), (17,33)],
        ["walking","E",6,3,.02,True, (23,45), (30,33)],
        ["walking","N",9,3,.02,True, (23,45), (23,33)],
        ["dying",  "N",12,3,0.01,False, (23,45)]
    ], 48, 48, mc_mask )
    
    mc.changeState("stopped")
    mc.AI = core.handlePlayerControls
    core.render_group.add(mc)
    
    mc.attacks = [ 
        featheratk.oBasicAttack(core.load_image("debug_feather.png", directory="gfx"))
    ]
    mc.use_attack = 0
    
    # load map
    load_level(level, mc)
    
    killed_bugs = 0
    lost_lives = 0
    mid_rank = 0
    rank_num = 0
    
    forceUpdate = True
    
    
    while True:
        lag = core.clock.tick(core.MAX_FPS)
        if lag <= 0:
            lag = 1
        
        level_time.update(level_time.value - lag/1000.0)
        if level_time.value < 0:
            main_char_death(mc)
            core.play_sound("mc_dying")
            lives.update(lives.value-1)
            lost_lives += 1
        
        if core.handleEvents() == "quit":
            return
                        
        dirty_rects = core.render_group.update(lag)
        core.level.updateNests(lag)
        exp_bg.animFrame = core.mathExp.state
        
        # check collisions
        # bugs
        if core.render_group.checkCollision(False, False, mc, core.baddys_group):
            main_char_death(mc)
            core.play_sound("mc_dying")
            lives.update(lives.value-1)
            lost_lives += 1
                
        # feather attacks
        kills = core.attack_group.checkCollision(False, False, None, core.baddys_group)
        score.update(score.value + 100*len(kills))
        killed_bugs += len(kills)
        for i in kills:
            core.play_sound("bug_dying")
            if hasattr(i[0], "attackMgr"):
                feather, bug = i
            if hasattr(i[1], "attackMgr"):
                bug, feather = i
            feather.attackMgr.kills += 1
            feather.die()
            bug.OnAnimationEnd = bug.die
            bug.direction = "N"
            bug.changeState("dying")
        

        if lives.value == 0 and mc.OnAnimationEnd is None:
            if rank_num == 0:
                _rank = 0
            else:
                _rank = mid_rank / rank_num
            core.next_screen = core.curry(game_over, score.value, _rank)
            return
        
        # draw level / sprites
        core.screen.blit(sky_bg, (0,0))
        dirty_rects.extend( core.level.draw(core.screen) )
        core.render_group.draw(core.screen)
        
        # draw stats
        # lives
        dirty_rects.extend( lives.draw(core.screen) )
        # math exp
        dirty_rects.extend( core.mathExp.draw(core.screen) )
        # time
        dirty_rects.extend( level_time.draw(core.screen) )
        # score
        dirty_rects.extend( score.draw(core.screen) )
        # attacks
        for i, attack in enumerate(mc.attacks):
            attack.update(lag)
            dirty_rects.extend( attack.draw(core.screen, i == mc.use_attack) )
                
        dirty_rects.append( core.fonts.Draw(core.screen, None, 24,
         str(core.clock.get_fps()), (650, 50), (255,255,255,255)))
        
        
        if forceUpdate:
            dirty_rects = None
        forceUpdate = False
        core.updateScreen(dirty_rects)
        

        
        # clear level =D
        if core.level.portalActive:
            _score, rank = level_clear_screen(mc, killed_bugs, level_time.value, lives.value, lost_lives, score.value)
            score.update(_score)
            mid_rank += rank
            rank_num += 1
            lost_lives = 0
            core.baddys_group.clear()
            core.attack_group.clear()
            level += 1
            if not load_level(level, mc):
                # TODO: ending sorta thingy
                core.next_screen = core.curry(hiscores_screen, score.value, mid_rank/rank_num)
                return
            
            
            level_time.update(core.level.time)
            mc.changeState("stopped")
            for attack in mc.attacks:
                attack.freeze = 0
                attack.used = 0
                attack.kills = 0
                attack.last_rects = []
            
            forceUpdate = True
        
        
def game_over(score, rank):
    core.play_music("game over", queue=True)
    
    # load game over screen
    goscreen = core.load_image("game_over.png", directory="gfx", auto_alpha=False)
    core.dissolve_transition( core.screen.copy().convert(), goscreen.copy().convert(), 1000 )
    
    while True:
        core.clock.tick(core.MAX_FPS)
        
        events = pygame.event.get()
        
        if core.handleEvents(events) == "quit":
            exit()
            
        for event in events:
            if event.type == KEYUP:
                core.next_screen = core.curry(hiscores_screen, score, rank)
                Input.update()
                return
        
        core.screen.blit(goscreen, (0,0))                        
        core.updateScreen()
    
def credits_screen():
    goscreen = core.load_image("credits.png", directory="gfx", auto_alpha=False)
    core.dissolve_transition( core.screen.copy().convert(), goscreen.copy().convert(), 1000 )
    
    while True:
        core.clock.tick(core.MAX_FPS)
        
        events = pygame.event.get()
        
        if core.handleEvents(events) == "quit":
            exit()
            
        for event in events:
            if event.type == KEYUP:
                core.play_sound("button_click")
                Input.update()
                core.next_screen = main_menu
                return
        
        core.screen.blit(goscreen, (0,0))                        
        core.updateScreen()
    
def yesno_prompt(text):
    last_screen = core.screen.copy().convert()
    ls = last_screen.copy().convert()
    bg = core.load_image("yesno_prompt.png", directory="gfx")
    selected = 0
    button_group = sprite.oSpriteGroup()
    def genButton(images, pos):
        button = sprite.oSprite()
        button.x = pos[0]
        button.y = pos[1]
        button.animations = {
            "active": {
                "N": [0,True,[sprite.oFrame(images[0])]]
            },
            "inactive": {
                "N": [0,True,[sprite.oFrame(images[1])]]
            }
        }
        button_group.add(button)
        return button
    
    selected = 0
    buttons = []
    buttons.append( genButton(core.game_images["no"], (200,300)) )
    buttons.append( genButton(core.game_images["yes"], (400,300)) )
    forceUpdate = True
    
    while True:
        lag = core.clock.tick(core.MAX_FPS)
        
        if core.handleEvents() == "quit":
            exit()
                    
        if Input.up(K_LEFT):
            core.play_sound("cursor_move")
            selected -= 1
            if selected < 0:
                selected = len(buttons)-1
        if Input.up(K_RIGHT):
            core.play_sound("cursor_move")
            selected += 1
            if selected >= len(buttons):
                selected = 0
        if Input.up(K_RETURN):
            core.play_sound("button_click")
            core.dissolve_transition(core.screen.copy().convert(), ls, 1000)
            return bool(selected)
        
        for i, button in enumerate(buttons):
            button.changeState("active" if i == selected else "inactive")
                    
        dirty_rects = button_group.update(lag)
        core.screen.blit(ls, (0,0))
        core.screen.blit(bg, (0,0))
        button_group.draw(core.screen)        
        
        dirty_rects.append(core.fonts.Draw(core.screen, None, 36, text,
         pygame.Rect(50,250,700,50), (255,255,255,255), alignHoriz='center', antialias=True))        
        
        if last_screen is not None:
            core.dissolve_transition(last_screen, core.screen.copy().convert(), 1000)
            last_screen = None
        
        core.updateScreen(dirty_rects)
 
def exit_screen():
    if yesno_prompt("Exit RED -spider lily-?"):
        core.next_screen = None
        return
    else:
        core.next_screen = main_menu
        return
    
def main_menu():
    core.play_music("main menu")
    # load bg
    bg = core.load_image("main_menu.png", directory="gfx", auto_alpha=False)
    selected = 0
    
    def genButton(images, pos):
        button = sprite.oSprite()
        button.x = pos[0]
        button.y = pos[1]
        button.animations = {
            "active": {
                "N": [0,True,[sprite.oFrame(images[0])]]
            },
            "inactive": {
                "N": [0,True,[sprite.oFrame(images[1])]]
            }
        }
        core.render_group.add(button)
        return button
    
    selected = 0
    buttons = []
    core.render_group.clear()
    for i, button in enumerate(["new game","help","credits","exit"]):
        y = 180 + i*40
        x = 75
        buttons.append( genButton(core.game_images[button], (x,y)) )
    
    menu_screens = [
        main_game,      # new game
        hiscores_screen,# high scores
        credits_screen, # credits
        exit_screen,    # exit
    ]
    
    forceUpdate = True
    last_screen = core.screen.copy().convert()
    
    while True:
        lag = core.clock.tick(core.MAX_FPS)
        
        if core.handleEvents() == "quit":
            exit()
                    
        if Input.up(K_UP):
            core.play_sound("cursor_move")
            selected -= 1
            if selected < 0:
                selected = len(buttons)-1
        if Input.up(K_DOWN):
            core.play_sound("cursor_move")
            selected += 1
            if selected >= len(buttons):
                selected = 0
        if Input.up(K_RETURN):
            core.next_screen = menu_screens[selected]
            core.play_sound("button_click")
            return
        
        for i, button in enumerate(buttons):
            button.changeState("active" if i == selected else "inactive")
                    
        dirty_rects = core.render_group.update(lag)
        core.screen.blit(bg, (0,0))
        core.render_group.draw(core.screen)        
        
        if last_screen is not None:
            core.dissolve_transition(last_screen, core.screen.copy().convert(), 1000)
            last_screen = None
        
        if forceUpdate:
            dirty_rects = None
        core.updateScreen(dirty_rects)
    
    
    
    