#!/usr/bin/env python
# -*- coding: utf_8 -*-
#
# @author  killdream <killdreamxd@gmail.com>
# @version 0.2.1 alpha
#
# Copyright (C) 2008 Quildreen <killdreamxd@gmail.com>
# -----                                                   
# This program is free software; you can redistribute  
# it and/or modify it under the terms of the GNU       
# General Public License as published by the Free      
# Software Foundation; either version 2 of the License,
# or (at your option) any later version.               
#                                                         
# This program is distributed in the hope that it will 
# be useful, but WITHOUT ANY WARRANTY; without even    
# the implied warranty of MERCHANTABILITY or FITNESS   
# FOR A PARTICULAR PURPOSE. See the GNU General Public 
# License for more details.                            
#                                                         
# You should have received a copy of the GNU General   
# Public License along with this program; if not,      
# write to the Free Software Foundation, Inc., 675     
# Mass Ave, Cambridge, MA 02139, USA.

# Import pygame modules
import pygame
from pygame.locals import *

import random
random.seed()

# Import some utility modules
import math
import core
import sprite

class oTile:
    def __init__(self, iImageID, iHeight, iWalkable, **kwargs):
        """Initializes oTile class"""
        self.imageID = iImageID
        self.height  = iHeight
        self.walkable= iWalkable
        self.changed = True
        self.portal  = kwargs.pop("portal" , False)
        self.active  = kwargs.pop("active" , False)
        self.number  = kwargs.pop("sNumber", -1)
        self.op      = kwargs.pop("op"     , -1)
        self.nest    = kwargs.pop("nest"   , None)
        
        
class oNest:
    def __init__(self, capacity, mapID, direction, mapa, monsGroup, **kwargs):
        self.capacity = capacity
        self.bugs     = []
        self.group    = monsGroup
        self.mapID    = mapID
        self.direction= direction
        self.mapa     = mapa
        self.state    = -1
        self.maxState = kwargs.pop("maxState", 1)
        self.unactive = 0
        self.openWork = 0
        self.damage   = 0
        self.mapa.mapData[self.mapID].nest = self
        self.changeState(kwargs.pop("state", 0))
        
    def changeState(self, newState):
        if newState != self.state:
            self.state = newState
            self.unactive = 0
            self.openWork = 0
            self.damage = 0
            self.mapa.mapData[self.mapID].changed = True
            self.mapa.genBitmask()
        
    def doDamage(self, amount):
        if self.state == 0:
            return
        
        self.damage += amount
        if self.damage > self.state * 2:
            self.changeState(self.state - 1)
            
                
        
    def update(self, lag):
        for bug in self.bugs:
            if bug.dead:
                self.bugs.remove(bug)
        
        if self.state < self.maxState:
            self.openWork += lag
            open_prob = 0
            if self.openWork > 6000:
                open_prob = 1
            if self.openWork > 15000:
                open_prob = 100
                
            if random.randint(0,100) < open_prob:
                self.changeState(self.state + 1)
                self.openWork = 0
        
        if self.state <= 0:
            return
        
        if len(self.bugs) < (self.capacity*self.state/2):
            self.unactive += lag
            if self.unactive > 5000:
                if random.randint(0,100) < self.state*self.state*10:
                    # create a new bug...
                    self.mapa.created_bugs += 1
                    bug = sprite.oSprite()
                    tw, th = self.mapa.tileSize
                    bug.x = (self.mapID % self.mapa.width)
                    bug.y = (self.mapID / self.mapa.width)
                    bug.speed = .1
                    bug.direction = self.direction
                    if bug.direction == "N":
                        bug.y -= 1
                    elif bug.direction == "W":
                        bug.x -= 1
                    elif bug.direction == "S":
                        bug.y += 1
                    elif bug.direction == "E":
                        bug.x += 1
                    bug.AI = sprite.bugAI
                    bug_sprite = core.load_image("bug.png", directory="gfx")
                    bug.animFromStrip( bug_sprite, [
                        ["stopped","E",0,1,0,True],
                        ["stopped","N",4,1,0,True],
                        ["stopped","W",8,1,0,True],
                        ["stopped","S",12,1,0,True],
                        ["dying",  "N",1,3,.01,False],
                        
                    ], 48, 48, bug_sprite)
                    bug.changeState("stopped")
                    bug.x = bug.x * tw + bug.hotspot()[0]
                    bug.y = bug.y * th + bug.hotspot()[1] + 33
                    
                    self.group.add(bug)
                    core.render_group.add(bug)
                    self.bugs.append(bug)
                    self.unactive = 0
    
class oMap:
    def __init__(self, **kwargs):
        """Initializes oMap class."""
        self.width    = kwargs.pop("width" , 0)
        self.height   = kwargs.pop("height", 0)
        self.mapData  = kwargs.pop("mapData", [])
        self.tileset  = kwargs.pop("tileset", [
            core.load_image("plain_block.png", directory="gfx/tileset"),    # 0
            core.load_image("brown_block.png", directory="gfx/tileset"),    # 1
            core.load_image("dirty_block.png", directory="gfx/tileset"),    # 2
            core.load_image("stone_block.png", directory="gfx/tileset"),    # 3
            core.load_image("grass_block.png", directory="gfx/tileset"),    # 4
            core.load_image("water_block.png", directory="gfx/tileset"),    # 5
            core.load_image("wall_block.png", directory="gfx/tileset"),     # 6
            core.load_image("wood_block.png", directory="gfx/tileset"),     # 7
            core.load_image("portal_inactive.png", directory="gfx/tileset"),# 8
            core.load_image("portal_active.png", directory="gfx/tileset"),  # 9
            core.load_image("star_0.png", directory="gfx/tileset"),         # 10
            core.load_image("star_1.png", directory="gfx/tileset"),         # 11
            core.load_image("star_2.png", directory="gfx/tileset"),         # 12
            core.load_image("star_3.png", directory="gfx/tileset"),         # 13
            core.load_image("star_4.png", directory="gfx/tileset"),         # 14
            core.load_image("star_5.png", directory="gfx/tileset"),         # 15
            core.load_image("star_6.png", directory="gfx/tileset"),         # 16
            core.load_image("star_7.png", directory="gfx/tileset"),         # 17
            core.load_image("star_8.png", directory="gfx/tileset"),         # 18
            core.load_image("star_9.png", directory="gfx/tileset"),         # 19
            core.load_image("mult.png", directory="gfx/tileset"),           # 20
            core.load_image("add.png", directory="gfx/tileset"),            # 21
            core.load_image("sub.png", directory="gfx/tileset"),            # 22
            core.load_image("div.png", directory="gfx/tileset"),            # 23
            core.load_image("mod.png", directory="gfx/tileset"),            # 24
            core.load_image("star_0a.png", directory="gfx/tileset"),        # 25
            core.load_image("star_1a.png", directory="gfx/tileset"),        # 26
            core.load_image("star_2a.png", directory="gfx/tileset"),        # 27
            core.load_image("star_3a.png", directory="gfx/tileset"),        # 28
            core.load_image("star_4a.png", directory="gfx/tileset"),        # 29
            core.load_image("star_5a.png", directory="gfx/tileset"),        # 30
            core.load_image("star_6a.png", directory="gfx/tileset"),        # 31
            core.load_image("star_7a.png", directory="gfx/tileset"),        # 32
            core.load_image("star_8a.png", directory="gfx/tileset"),        # 33
            core.load_image("star_9a.png", directory="gfx/tileset"),        # 34
            core.load_image("active.png",  directory="gfx/tileset"),        # 35
            core.load_image("tree_ugly.png", directory="gfx/tileset"),      # 36
            core.load_image("tree.png", directory="gfx/tileset"),           # 37
            core.load_image("tree_tall.png", directory="gfx/tileset")       # 38
         ])
        self.x        = kwargs.pop("x", 0)
        self.y        = kwargs.pop("y", 0)
        self.tileSize = kwargs.pop("tileSize", (64,50))
        self.portalN  = kwargs.pop("portal", 0)
        self.portalActive = False
        self.created_bugs = 0
        self.time     = kwargs.pop("time", 300)
        self.nests    = [ ]
        self.genBitmask()
    
    def draw(self, surface):
        """Draw map data onto given surface."""
        sw, sh = surface.get_size()
        tw, th = self.tileSize
        hTiles = int(math.ceil(sh/th))
        wTiles = int(math.ceil(sw/tw))
        
        dirty = []
        
        # iter over the map and draw it onscreen =D
        for y in range(hTiles+1):
            if y >= self.height:
                break
            for x in range(wTiles+1):
                if x >= self.width:
                    break
                tile = self.mapData[y*self.width + x]
                
                tx = x*tw + self.x
                ty = y*th + self.y
                
                if tile is None:                        
                    continue
                
                r = [ ]
                if tile.number == -1 or (tile.number > -1 and (tile.op > -1 or tile.active)):
                    r.append( surface.blit(self.tileset[tile.imageID], (tx, ty)) )
                # if portal
                if tile.portal:
                    img = self.tileset[9] if tile.active else self.tileset[8]
                    r.append( surface.blit(img, (tx, ty-tile.height)) )
                # if number
                if tile.number > -1:
                    n = 25 if tile.active else 10
                    surface.blit(self.tileset[n+tile.number], (tx, ty-tile.height))
                    r.append( pygame.Rect(tx, ty, 64, 108) )
                # if op
                if tile.op > -1:
                    r.append( surface.blit(self.tileset[20+tile.op], (tx, ty-tile.height)) )
                              
                # if nest
                if tile.nest is not None:
                    r.append( surface.blit(self.tileset[36+tile.nest.state], (tx, ty-tile.height)) )
                            
                if tile.portal or tile.op > -1:
                    if tile.active:
                        surface.blit(self.tileset[35], (tx, ty-tile.height))
                    r.append( pygame.Rect(tx, ty-tile.height, 64, 108) )
                    
                if tile.changed:
                    dirty.extend(r)
                    tile.changed = False
                    
        return dirty
                    
                    
    def getTile(self, x, y):
        if y*self.width+x >= len(self.mapData):
            return None
        
        tile = self.mapData[y*self.width + x] or oTile(0,0,False)
        if tile.number == -1 or (tile.number > -1 and (tile.op > -1 or tile.active)):
            return tile
        else:
            return None
        
    def findPortal(self):
        for index, tile in enumerate(self.mapData):
            if tile is not None and tile.portal:
                x = int(index % self.width)
                y = int(index / self.width)
                return x,y
        return 0, 0
        
    def activateTiles(self, r):
        for tile in self.mapData:
            if tile is not None:
                if tile.number >-1 and tile.op == -1:
                    tile.active = tile.number == r
                    tile.changed = True
                if tile.portal:
                    tile.active = self.portalN == r
                    tile.changed = True
                    
    def updateNests(self, lag):
        for nest in self.nests:
            nest.update(lag)
        
    def walkable(self, x, y, mask=None):
        x = int(x)
        y = int(y)
        if mask is not None:
            return self.bitmask.overlap(mask, (x,y)) is None
        elif self.bitmask.get_at((x,y)) > 0:
            return False
        
        return True
        
    def genBitmask(self):
        tw, th = self.tileSize
        bmSurf = pygame.Surface((max(tw*self.width, 800), max(th*self.height, 600)))
        bmSurf.fill((255,255,255,255))
        bmSurf.set_colorkey((0,0,0,255))
        for y in range(self.height):
            for x in range(self.width):
                tile = (self.getTile(x,y) or oTile(0,0,False))
                if tile.walkable and tile.nest is None:
                    bmSurf.fill((0,0,0,255), pygame.Rect(x*tw+self.x,y*th+33+self.y,tw,th))

        self.bitmask = pygame.mask.from_surface(bmSurf)            
        
    def save(self, filename):
        import xdrlib
        p = xdrlib.Packer()
        p.pack_string("RSL-MAP")
        p.pack_int(self.width)
        p.pack_int(self.height)
        p.pack_int(self.portalN)
        for tile in range(len(self.mapData)):
            if self.mapData[tile] is None:
                p.pack_int(-1)
            else:
                p.pack_int(self.mapData[tile].imageID)
                p.pack_int(self.mapData[tile].height)
                p.pack_bool(self.mapData[tile].walkable)
                p.pack_bool(self.mapData[tile].portal)
                p.pack_int(self.mapData[tile].number)
                p.pack_int(self.mapData[tile].op)
            
        f = open(filename, "wb")
        f.write(p.get_buffer())
        f.close()
        
    def load(self, filename):
        f = open(filename, "rb")
        data = f.read()
        f.close()
        
        import xdrlib
        p = xdrlib.Unpacker(data)
        if p.unpack_string() == "RSL-MAP":
            self.width = p.unpack_int()
            self.height = p.unpack_int()
            self.portalN = p.unpack_int()
            self.mapData = [ None for i in range(self.width*self.height) ]
            for i in range(self.width*self.height):
                id = p.unpack_int()
                if id == -1:
                    self.mapData[i] = None
                else:
                    self.mapData[i] = oTile(id, p.unpack_int(), p.unpack_bool())
                    self.mapData[i].portal = p.unpack_bool()
                    self.mapData[i].number = p.unpack_int()
                    self.mapData[i].op = p.unpack_int()
        
        self.portalActive = False
        self.created_bugs = 0
        self.genBitmask()
            
